#include <OneWire.h>
#include <DallasTemperature.h>

#define ONE_WIRE_BUS 2
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

const int triggerPin = 9;
const int echoPin = 10;
const int ultrasonicSource = 11;
const int temperaturePin = 12;
const int switchReadPin = A0;
const int LEDGreenPin = 8;
const int LEDRedPin = 7;
const int temperaturePower = 13;
const int LEDBluePin = 4;


long duration;
int distance;
float switchVal;
bool startHeating;
bool startKeepWarm;
bool sleepState;
bool warmInitial;
unsigned long time;
unsigned long lastTime;
unsigned long sleepTime;
unsigned long warmTime;
unsigned long sleepTrackTime;
unsigned long currentTemperature;

void setup() {
    time = 0;
    lastTime = 0;
    sleepTime = 0;
    warmTime = 0;
    switchVal = 0.0f;
    currentTemperature = 0;
    startHeating = false;
    warmInitial = true;

    pinMode(switchReadPin, INPUT);
    pinMode(echoPin, INPUT);
    pinMode(triggerPin, OUTPUT);
    pinMode(ultrasonicSource, OUTPUT);
    pinMode(temperaturePin, OUTPUT);
    pinMode(LEDGreenPin, OUTPUT);
    pinMode(LEDRedPin, OUTPUT);
    pinMode(LEDBluePin, OUTPUT);
    pinMode(temperaturePower, OUTPUT);

    Serial.begin(9600);
    sensors.begin();
}

void loop() {

      switchVal = analogRead(switchReadPin);
      Serial.print("switchVal: "); Serial.println(switchVal);

      digitalWrite(temperaturePower, HIGH);
      sensors.requestTemperatures();
      currentTemperature = sensors.getTempCByIndex(0);
      Serial.print("current temperature: ");Serial.println(currentTemperature);

      time = millis();
      digitalWrite(ultrasonicSource, HIGH);
      digitalWrite(triggerPin, LOW);
      delayMicroseconds(2);
      digitalWrite(triggerPin, HIGH);
      delayMicroseconds(10);
      digitalWrite(triggerPin, LOW);
      duration = pulseIn(echoPin, HIGH);
      distance= duration*0.034/2;
      //Serial.print("distance: ");Serial.println(distance);

      if (switchVal > 512.0) {
          sleepTrackTime = 0.0;
          

          if (distance < 8 && time - lastTime > 800) {
              lastTime = time;
              if (!startHeating && !startKeepWarm && !sleepState) {
                  startHeating = true;
              } else if (startHeating && !startKeepWarm && !sleepState) {
                  startHeating = false;
                  startKeepWarm = true;
                  warmTime = time;
                  warmInitial = true;
              } else if (!startHeating && startKeepWarm && !sleepState) {
                  startKeepWarm = false;
                  sleepTime = time;
                  sleepState = true;
              } else if (!startHeating && !startKeepWarm && sleepState && time - sleepTime > 5000) {
                  sleepState = false;
                  startHeating = true;
              } else {
                  sleepState = false;
              }
          }

      } else {
            if (startKeepWarm && sleepTrackTime == 0.0) {
                sleepTrackTime = millis();
            }
            
            if (currentTemperature < 27.0) {
                startHeating = true;
                sleepTrackTime = 0.0;
                startKeepWarm = false;
                sleepState = false;
            }
            if (currentTemperature >= 27.0 && currentTemperature <= 34.0) {
                startHeating = false;
                startKeepWarm = true;
                sleepState = false;
            }
//            if (millis() - sleepTrackTime >= 5000.0 && sleepTrackTime != 0.0) {
//                Serial.print("sleepTrackTime: ");Serial.println(sleepTrackTime);
//                startHeating = false;
//                startKeepWarm = false;
//                sleepState = true;
//            }
      }

      if (!startHeating && !startKeepWarm && !sleepState) {
         // Serial.println("I am shutted down");
          digitalWrite(LEDGreenPin, LOW);
          digitalWrite(LEDBluePin, LOW);
          digitalWrite(LEDRedPin, LOW);
      }

      if (startHeating) {
          //Serial.println("in start heating stage");
          digitalWrite(temperaturePin, HIGH);
          digitalWrite(LEDGreenPin, HIGH);
          digitalWrite(LEDBluePin, LOW);
          digitalWrite(LEDRedPin, LOW);
      }

      if (startKeepWarm) {
          //Serial.println("in keep warm stage");
//          if (time - warmTime <= 5000) {
//              warmInitial = true;
//              digitalWrite(temperaturePin, HIGH);
//          } else {
//              if (warmInitial) {
//                  warmTime = time;
//                  warmTime = false;
//              }
//              digitalWrite(temperaturePin, LOW);
//          }
          if (currentTemperature < 20.0) {
                startHeating = true;
                //sleepTrackTime = 0.0;
                startKeepWarm = false;
                sleepState = false;
          }
          digitalWrite(temperaturePin, LOW);
          digitalWrite(LEDGreenPin, LOW);
          digitalWrite(LEDBluePin, HIGH);
          digitalWrite(LEDRedPin, LOW);
      }

      if (sleepState) {
          digitalWrite(LEDGreenPin, LOW);
          digitalWrite(LEDBluePin, LOW);
          digitalWrite(LEDRedPin, HIGH);
          digitalWrite(temperaturePin, LOW);
      }
}
